﻿package {
	
//- IMPORTS -----------------------------------------------------------------------------------------------
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldType;
	import flash.text.TextFieldAutoSize;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.net.FileReference;
	import flash.net.FileFilter;
	import flash.geom.Rectangle;
	import flash.display.Loader;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.MovieClip;
	import flash.text.TextFormat;
	import flash.filters.DropShadowFilter;
	import com.greensock.TweenLite;
	import com.oskarwicha.images.FaceDetection.FaceDetector;
	import com.oskarwicha.images.FaceDetection.Events.FaceDetectorEvent;
	
	public class FaceTagger extends Sprite {
	
//- PRIVATE & PROTECTED VARIABLES -------------------------------------------------------------------------

		private var statusTxt:TextField;
		private var fileRef:FileReference;
		private var fileFilter:FileFilter;
		private var loader:Loader;
		private var bitmap:Bitmap;
		private var image:MovieClip;
		private var darkBox:Sprite;
		private var detector:FaceDetector;
		private var currentTag:PhotoTag;
		
//- PUBLIC & INTERNAL VARIABLES ---------------------------------------------------------------------------
		
		public static const MIN_WIDTH:Number = 50;
		public static const MIN_HEIGHT:Number = 50;
		public static const MAX_WIDTH:Number = 1000;
		public static const MAX_HEIGHT:Number = 1000;
		public static const FILE_TYPES:String = "*.jpg; *.jpeg; *.png";
		
//- CONSTRUCTOR	-------------------------------------------------------------------------------------------
	
		public function FaceTagger() {
			
			statusTxt = new TextField();
			fileRef = new FileReference();
			fileFilter = new FileFilter( "Image (" + FILE_TYPES + ")", FILE_TYPES );
			loader = new Loader();
			detector = new FaceDetector();
			darkBox = new Sprite();
			init();
		}

//- PRIVATE & PROTECTED METHODS ---------------------------------------------------------------------------
		
		private function init():void {
			
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			stage.addEventListener( Event.RESIZE, onStageResize );
			browseBtn.addEventListener( MouseEvent.CLICK, browse );
			addBtn.addEventListener( MouseEvent.CLICK, addTag );
			fileRef.addEventListener( Event.SELECT, onFileSelected );
			fileRef.addEventListener( Event.COMPLETE, onFileComplete );
			loader.contentLoaderInfo.addEventListener( Event.COMPLETE, detectFaces );
			
			statusTxt.type = TextFieldType.DYNAMIC;
			statusTxt.selectable = false;
			statusTxt.autoSize = TextFieldAutoSize.CENTER;
			statusTxt.defaultTextFormat = new TextFormat( null, 22, 0xFFFFFF, true );
			statusTxt.text = "Choose an image to upload";
			statusTxt.filters = [ new DropShadowFilter( 5, 45, 0, 1, 5, 5, 1, 3 ) ];
			
			darkBox.graphics.beginFill( 0, .5 );
			darkBox.graphics.drawRect( 0, 0, stage.stageWidth, stage.stageHeight );
			darkBox.visible = false;
			darkBox.addEventListener( MouseEvent.CLICK, exitEditMode );
			
			addChild( statusTxt );
			addChild( darkBox );
			
			positionContents();
		}
		
		private static function inRange( width:Number, height:Number ):Boolean {
			
			if ( width < MIN_WIDTH || width > MAX_HEIGHT ) {
				
				return false;
			}
			else if (  height < MIN_HEIGHT || height > MAX_HEIGHT ) {
				
				return false;
			}
			else {
				
				return true;
			}
		}
		
		private function positionContents():void {
			
			browseBtn.x = stage.stageWidth - browseBtn.width - 10; // Positions the 'Browse' button 10 pixels from the top right of the stage
			addBtn.x = browseBtn.x - addBtn.width - 10; // Positions the 'Add Tag' button 10 pixels to the left of the 'Browse' button
			statusTxt.x = ( stage.stageWidth - statusTxt.width ) / 2; // Horizontally center the status textfield to the stage's 
			statusTxt.y = stage.stageHeight - statusTxt.height - 10; // Positions the textfield 10 pixels from the bottom of the stage
			darkBox.width = stage.stageWidth;
			darkBox.height = stage.stageHeight;
			// If the Bitmap object exists center it to the stage
			if ( image ) {
				
				image.x = ( stage.stageWidth - image.width ) / 2;
				image.y = ( stage.stageHeight - image.height ) / 2;
			}
		}
		
		/*
		 *
		 * @method	Returns a Rectangle object from a string containing the location and size of
		 *			of a cropped face. The string comes from an array from the FaceDetector object's
		 *			ObjectDetector object.
		 *
		 */
		
		private function newTag( rectangle:Rectangle ):void {
			
			var tag:PhotoTag = new PhotoTag( rectangle, image );
			image.addChild( tag );
			tag.addEventListener( MouseEvent.ROLL_OVER, onTagRollOver );
			tag.addEventListener( MouseEvent.DOUBLE_CLICK, enterEditMode );
			tag.addEventListener( Event.REMOVED, onTagRemoved );
		}
		
//- PUBLIC & INTERNAL METHODS -----------------------------------------------------------------------------
	
		// ** No Public or Internal Methods **
	
//- EVENT HANDLERS ----------------------------------------------------------------------------------------
		
		private function onStageResize( e:Event ):void {
			
			positionContents();
		}
		
		private function browse( e:MouseEvent ):void {
			
			fileRef.browse( [ fileFilter ] );
		}
		
		private function addTag( e:MouseEvent ):void {
			
			// If the bitmap exists add a new tag to the stage
			if ( image ) {
				
				var rect:Rectangle = new Rectangle( 0, 0, 75, 75 );
				newTag( rect );
			}
		}
		
		private function onFileSelected( e:Event ):void {
			
			browseBtn.enabled = false;
			statusTxt.text = "loading";
			fileRef.load();
		}
		
		private function onFileComplete( e:Event ):void {
			
			loader.loadBytes( fileRef.data );
		}
		
		private function detectFaces( e:Event ):void {
			
			bitmap = Bitmap( loader.content );
			
			if ( !inRange( bitmap.width, bitmap.height ) ) {
				
				if ( !image ) image = new MovieClip();
				image.addChild( bitmap );
				addChildAt( image, 0 );
				image.alpha = 0;
				TweenLite.to( image, 1, { alpha:1 } );
				statusTxt.text = "image to large for face detection";
				return;
			}
			
			statusTxt.text = "detecting faces... please wait";
			
			// Listen for FaceDetectorEvents and begin face detection
			detector.addEventListener( FaceDetectorEvent.FACE_CROPED, onFacesDetected );
			detector.addEventListener( FaceDetectorEvent.NO_FACES_DETECTED, onNoFaces );
			detector.loadFaceImageFromBitmap( bitmap );
		}
		
		/*
		 *
		 * @param	IMPORTANT!: Do not attempt to set the Event parameter to a FaceDetectorEvent
		 *			object. You will get an error.
		 *
		 */
		private function onFacesDetected( e:Event ):void {
			
			var faces:Array = detector.objectDetector.detected;
			if ( !image ) image = new MovieClip()
			else return;
			image.addChild( bitmap );
			addChildAt( image, 0 );
			positionContents();
			image.alpha = 0;
			TweenLite.to( image, 1, { alpha:1 } );
			statusTxt.text = "faces detected";
			
			for each( var face:Rectangle in faces ) {
				
				var rect:Rectangle = face;
				newTag( rect );
			}
			
			positionContents(); // To center the image
		}
		
		private function onNoFaces( e:Event ):void {
			
			statusTxt.text = "no faces were detected";
			if ( !image ) image = new MovieClip();
			image.addChild( bitmap );
			addChildAt( image, 0 );
			positionContents();
			image.alpha = 0;
			TweenLite.to( image, 1, { alpha:1 } );
		}
		
		private function onTagRollOver( e:MouseEvent ):void {
			
			if ( !e.target.editMode ) statusTxt.text = "double-click to edit";
		}
		
		private function enterEditMode( e:MouseEvent ):void {
			
			statusTxt.text = "click the background to exit 'edit' mode\n";
			statusTxt.appendText( "use the mouse wheel or the up and down\n" );
			statusTxt.appendText( "arrow keys to resize the tag" );
			
			var tag:PhotoTag = e.target as PhotoTag;
			darkBox.visible = true;
			addChild( darkBox );
			addChild( tag );
			addChild( statusTxt );
			tag.x = image.x + tag.rectangle.x;
			tag.y = image.y + tag.rectangle.y;
			tag.enterEditMode();
			currentTag = tag;
			positionContents();
		}
		
		private function exitEditMode( e:MouseEvent = null ):void {
			
			darkBox.visible = false;
			
			if ( currentTag ) {
				
				statusTxt.text = " ";
				currentTag.rectangle.x = currentTag.x - image.x;
				currentTag.rectangle.y = currentTag.y - image.y;
				currentTag.exitEditMode();
				currentTag = null;
				positionContents();
			}
		}
		
		private function onTagRemoved( e:Event ):void {
			
			if ( e.target.removed ) {	
				
				exitEditMode();
			}
		}
	
//- GETTERS & SETTERS -------------------------------------------------------------------------------------
	
		// ** No Getters or Setters **
	
//- HELPERS -----------------------------------------------------------------------------------------------
	
		// ** No Helpers **
	
//- END CLASS ---------------------------------------------------------------------------------------------

	}
}
