﻿package {
	
//- IMPORTS -----------------------------------------------------------------------------------------------
	
	import flash.display.Sprite;
	import flash.geom.Rectangle;
	import flash.events.Event;
	import flash.display.MovieClip;
	import flash.filters.DropShadowFilter;
	import flash.events.MouseEvent;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	import com.greensock.TweenLite;
	
	public class PhotoTag extends Sprite {
	
//- PRIVATE & PROTECTED VARIABLES -------------------------------------------------------------------------

		private var _rectangle:Rectangle;
		private var _image:MovieClip;
		private var _editMode:Boolean;
		private var _removed:Boolean;
		private var removeBtn:RemoveButton;
		
//- PUBLIC & INTERNAL VARIABLES ---------------------------------------------------------------------------
		
		public static const LINE_THICKNESS:Number = 6;
		public static const LINE_COLOR:uint = 0x00CCFF;
		public static const FILL_COLOR:uint = 0xFFFFFF;
		public static const ELLIPSE_WIDTH:Number = 10;
		public static const ELLIPSE_HEIGHT:Number = 10;
		public static const INIT_ALPHA:Number = .6;
		
//- CONSTRUCTOR	-------------------------------------------------------------------------------------------
	
		public function PhotoTag( rectangle:Rectangle, image:MovieClip ) {
			
			_rectangle = rectangle;
			_image = image;
			removeBtn = new RemoveButton();
			graphics.lineStyle( LINE_THICKNESS, LINE_COLOR );
			graphics.beginFill( FILL_COLOR, .5 );
			graphics.drawRoundRect( 0, 0, rectangle.width, rectangle.height, ELLIPSE_WIDTH, ELLIPSE_HEIGHT );
			filters = [ new DropShadowFilter( 5, 45, 0, 1, 5, 5, 1, 3 ) ];
			alpha = INIT_ALPHA;
			_editMode = false;
			addEventListener( Event.ADDED, onAdded );
		}

//- PRIVATE & PROTECTED METHODS ---------------------------------------------------------------------------
		
		private function init():void {
			
			if ( this.parent != image ) _image.addChild( this );
			x = _rectangle.x;
			y = _rectangle.y;
			addChild( removeBtn );
			removeBtn.alpha = 0;
			addEventListener( MouseEvent.ROLL_OVER, overState );
			addEventListener( MouseEvent.MOUSE_DOWN, onMouseDown );
			stage.addEventListener( MouseEvent.MOUSE_WHEEL, onMouseWheel );
			stage.addEventListener( KeyboardEvent.KEY_DOWN, onKeyDownE );
			doubleClickEnabled = true;
			removeBtn.addEventListener( MouseEvent.CLICK, remove );
		}
		
		private function draw( newWidth:Number, newHeight:Number ):void {
			
			graphics.clear();
			graphics.lineStyle( LINE_THICKNESS, LINE_COLOR );
			graphics.beginFill( FILL_COLOR, .5 );
			graphics.drawRoundRect( 0, 0, newWidth, newHeight, ELLIPSE_WIDTH, ELLIPSE_HEIGHT );
		}
		
		private function resize( percent:Number ):void {
			
			var newWidth:Number = ( percent * _rectangle.width ) / 100 + _rectangle.width;
			var newHeight:Number = ( percent * _rectangle.height ) / 100 + _rectangle.height;
			
			if ( newWidth < 60 ) {
				
				newWidth = 60;
			}
			if ( newHeight < 60 ) {
				
				newHeight = 60;
			}
			
			var newX:Number = ( x - image.x ) - ( newWidth - _rectangle.width ) / 2;
			var newY:Number = ( y - image.y ) - ( newHeight - _rectangle.height ) / 2;
			
			_rectangle.x = newX;
			_rectangle.y = newY;
			_rectangle.width = newWidth;
			_rectangle.height = newHeight;
			draw( newWidth, newHeight );
			x = newX + image.x;
			y = newY + image.y;
		}
		
//- PUBLIC & INTERNAL METHODS -----------------------------------------------------------------------------
	
		public function enterEditMode():void {
			
			if ( !_editMode ) {
				
				_editMode = true;
				removeEventListener( MouseEvent.ROLL_OUT, outState );
				draw( _rectangle.width, _rectangle.height );
			}
		}
		
		public function exitEditMode():void {
			
			if ( _editMode && !removed ) {
				
				_editMode = false;
				image.addChild( this );
				x = _rectangle.x;
				y = _rectangle.y;
				stopDrag();
				alpha = INIT_ALPHA;
				TweenLite.to( removeBtn, .1, { alpha:0 } );
				addEventListener( MouseEvent.ROLL_OVER, overState );
			}
		}
	
//- EVENT HANDLERS ----------------------------------------------------------------------------------------
	
		private function onAdded( e:Event ):void {
			
			removeEventListener( Event.ADDED, onAdded );
			init();
		}
		
		private function overState( e:MouseEvent ):void {
			
			if ( _editMode ) return;
			removeEventListener( MouseEvent.ROLL_OVER, outState );
			
			removeBtn.visible = true;
			removeBtn.alpha = 0;
			
			var percent:Number = 40; // The percentage that we want to increase in size
			var newWidth:Number = ( percent * _rectangle.width ) / 100 + _rectangle.width;
			var newHeight:Number = ( percent * _rectangle.height ) / 100 + _rectangle.height;
			var newX:Number = _rectangle.x - ( newWidth - _rectangle.width ) / 2;
			var newY:Number = _rectangle.y - ( newHeight - _rectangle.height ) / 2;
			TweenLite.to( this, .1, { alpha:1, x:newX, y:newY, width:newWidth, height:newHeight } );
			TweenLite.to( removeBtn, .1, { alpha:1 } );
			addEventListener( MouseEvent.ROLL_OUT, outState );
		}
		
		private function outState( e:MouseEvent ):void {
			
			if ( _editMode ) return;
			removeEventListener( MouseEvent.ROLL_OUT, outState );
			TweenLite.to( this, .1, { alpha:INIT_ALPHA, x:_rectangle.x, y:_rectangle.y, width:_rectangle.width, height:_rectangle.height } );
			TweenLite.to( removeBtn, .1, { alpha:0 } );
			addEventListener( MouseEvent.ROLL_OVER, overState );
		}
		
		private function onMouseDown( e:MouseEvent ):void {
			
			if ( !_editMode ) return;
			removeEventListener( MouseEvent.MOUSE_DOWN, onMouseDown );
			var thisRect:Rectangle = image.getRect( this );
			startDrag( false, new Rectangle( image.x, image.y, thisRect.width - _rectangle.width, thisRect.height - _rectangle.height ) );
			addEventListener( MouseEvent.MOUSE_UP, onMouseUp );
		}
		
		private function onMouseUp( e:MouseEvent ):void {
			
			if ( !_editMode ) return;
			removeEventListener( MouseEvent.MOUSE_UP, onMouseUp );
			stopDrag();
			addEventListener( MouseEvent.MOUSE_DOWN, onMouseDown );
		}
		
		private function onMouseWheel( e:MouseEvent ):void {
			
			if ( !_editMode ) return;
			var percent:Number = e.delta * 10; // The percentage that we want to increase in size
			resize( percent );
		}
		
		private function onKeyDownE( e:KeyboardEvent ):void {
			
			if ( !_editMode ) return;
			
			var percent:Number;
			
			if ( e.keyCode == Keyboard.UP ) {
				
				percent = 10;
			}
			else if ( e.keyCode == Keyboard.DOWN ) {
				
				percent = -10;
			}
			else {
				
				return;
			}
			
			resize( percent );
		}
		
		private function remove( e:MouseEvent ):void {
			
			if ( this.parent ) {
				
				_removed = true;
				this.parent.removeChild( this );
			}
		}
	
//- GETTERS & SETTERS -------------------------------------------------------------------------------------
		
		public function get rectangle():Rectangle {
			
			return _rectangle;
		}
		
		public function get image():MovieClip {
			
			return _image;
		}
		
		public override function set width( value:Number ):void {
			
			draw( value, height );
		}
		
		public override function set height( value:Number ):void {
			
			draw( width, value );
		}
		
		public function get editMode():Boolean {
			
			return _editMode;
		}
		
		public function get removed():Boolean {
			
			return _removed;
		}
	
//- HELPERS -----------------------------------------------------------------------------------------------
	
		// ** No Helpers **
	
//- END CLASS ---------------------------------------------------------------------------------------------

	}
}
